==> spring-data-commons-1.8.2.RELEASE-sources.jar: org/springframework/data/querydsl/QSort.java <==
/*
 * Copyright 2013 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.springframework.data.querydsl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.springframework.data.domain.Sort;
import org.springframework.util.Assert;

import com.mysema.query.types.Expression;
import com.mysema.query.types.OrderSpecifier;

/**
 * Sort option for queries that wraps a querydsl {@link OrderSpecifier}.
 * 
 * @author Thomas Darimont
 */
public class QSort extends Sort implements Serializable {

	private static final long serialVersionUID = -6701117396842171930L;

	private final List<OrderSpecifier<?>> orderSpecifiers;

	/**
	 * Creates a new {@link QSort} instance with the given {@link OrderSpecifier}s.
	 * 
	 * @param orderSpecifiers must not be {@literal null} or empty;
	 */
	public QSort(OrderSpecifier<?>... orderSpecifiers) {
		this(Arrays.asList(orderSpecifiers));
	}

	/**
	 * Creates a new {@link QSort} instance with the given {@link OrderSpecifier}s.
	 * 
	 * @param orderSpecifiers must not be {@literal null} or empty;
	 */
	public QSort(List<OrderSpecifier<?>> orderSpecifiers) {
		super(toOrders(orderSpecifiers));
		Assert.notEmpty(orderSpecifiers, "Order specifiers must not be null or empty!");
		this.orderSpecifiers = orderSpecifiers;
	}

	/**
	 * Converts the given {@link OrderSpecifier}s into a list of {@link Order}s.
	 * 
	 * @param orderSpecifiers must not be {@literal null} or empty.
	 * @return
	 */
	private static List<Order> toOrders(List<OrderSpecifier<?>> orderSpecifiers) {

		Assert.notEmpty(orderSpecifiers, "Order specifiers must not be null or empty!");

		List<Order> orders = new ArrayList<Sort.Order>();
		for (OrderSpecifier<?> orderSpecifier : orderSpecifiers) {
			orders.add(toOrder(orderSpecifier));
		}

		return orders;
	}

	/**
	 * Converts the given {@link OrderSpecifier} into an {@link Order}.
	 * 
	 * @param orderSpecifier must not be {@literal null}.
	 * @return
	 */
	private static Order toOrder(OrderSpecifier<?> orderSpecifier) {

		Assert.notNull(orderSpecifier, "Order specifier must not be null!");

		Expression<?> target = orderSpecifier.getTarget();
		Object targetElement = ((com.mysema.query.types.Path<?>) target).getMetadata().getElement();

		Assert.notNull(targetElement, "Target element must not be null!");

		return new Order(targetElement.toString()).with(orderSpecifier.isAscending() ? Direction.ASC : Direction.DESC);
	}

	/**
	 * @return the orderSpecifier
	 */
	public List<OrderSpecifier<?>> getOrderSpecifiers() {
		return orderSpecifiers;
	}

	/**
	 * Returns a new {@link QSort} consisting of the {@link OrderSpecifier}s of the current {@code QSort} combined with
	 * the ones from the given {@code QSort}.
	 * 
	 * @param sort can be {@literal null}.
	 * @return
	 */
	public QSort and(QSort sort) {
		return sort == null ? this : and(sort.getOrderSpecifiers());
	}

	/**
	 * Returns a new {@link QSort} consisting of the {@link OrderSpecifier}s of the current {@link QSort} combined with
	 * the given ones.
	 * 
	 * @param orderSpecifiers must not be {@literal null} or empty.
	 * @return
	 */
	public QSort and(List<OrderSpecifier<?>> orderSpecifiers) {

		Assert.notEmpty(orderSpecifiers, "OrderSpecifiers must not be null or empty!");

		List<OrderSpecifier<?>> newOrderSpecifiers = new ArrayList<OrderSpecifier<?>>(this.orderSpecifiers);
		newOrderSpecifiers.addAll(orderSpecifiers);

		return new QSort(newOrderSpecifiers);
	}

	/**
	 * Returns a new {@link QSort} consisting of the {@link OrderSpecifier}s of the current {@link QSort} combined with
	 * the given ones.
	 * 
	 * @param orderSpecifiers must not be {@literal null} or empty.
	 * @return
	 */
	public QSort and(OrderSpecifier<?>... orderSpecifiers) {

		Assert.notEmpty(orderSpecifiers, "OrderSpecifiers must not be null or empty!");
		return and(Arrays.asList(orderSpecifiers));
	}
}

==> spring-data-commons-1.9.1.RELEASE-sources.jar: org/springframework/data/querydsl/QSort.java <==
/*
 * Copyright 2013-2014 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.springframework.data.querydsl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.springframework.data.domain.Sort;
import org.springframework.util.Assert;

import com.mysema.query.types.Expression;
import com.mysema.query.types.OrderSpecifier;
import com.mysema.query.types.Path;

/**
 * Sort option for queries that wraps a querydsl {@link OrderSpecifier}.
 * 
 * @author Thomas Darimont
 */
public class QSort extends Sort implements Serializable {

	private static final long serialVersionUID = -6701117396842171930L;

	private final List<OrderSpecifier<?>> orderSpecifiers;

	/**
	 * Creates a new {@link QSort} instance with the given {@link OrderSpecifier}s.
	 * 
	 * @param orderSpecifiers must not be {@literal null} or empty;
	 */
	public QSort(OrderSpecifier<?>... orderSpecifiers) {
		this(Arrays.asList(orderSpecifiers));
	}

	/**
	 * Creates a new {@link QSort} instance with the given {@link OrderSpecifier}s.
	 * 
	 * @param orderSpecifiers must not be {@literal null} or empty;
	 */
	public QSort(List<OrderSpecifier<?>> orderSpecifiers) {
		super(toOrders(orderSpecifiers));
		Assert.notEmpty(orderSpecifiers, "Order specifiers must not be null or empty!");
		this.orderSpecifiers = orderSpecifiers;
	}

	/**
	 * Converts the given {@link OrderSpecifier}s into a list of {@link Order}s.
	 * 
	 * @param orderSpecifiers must not be {@literal null} or empty.
	 * @return
	 */
	private static List<Order> toOrders(List<OrderSpecifier<?>> orderSpecifiers) {

		Assert.notEmpty(orderSpecifiers, "Order specifiers must not be null or empty!");

		List<Order> orders = new ArrayList<Sort.Order>();
		for (OrderSpecifier<?> orderSpecifier : orderSpecifiers) {
			orders.add(toOrder(orderSpecifier));
		}

		return orders;
	}

	/**
	 * Converts the given {@link OrderSpecifier} into an {@link Order}.
	 * 
	 * @param orderSpecifier must not be {@literal null}.
	 * @return
	 */
	private static Order toOrder(OrderSpecifier<?> orderSpecifier) {

		Assert.notNull(orderSpecifier, "Order specifier must not be null!");

		Expression<?> target = orderSpecifier.getTarget();
		Object targetElement = target instanceof Path ? ((com.mysema.query.types.Path<?>) target).getMetadata()
				.getElement() : target;

		Assert.notNull(targetElement, "Target element must not be null!");

		return new Order(targetElement.toString()).with(orderSpecifier.isAscending() ? Direction.ASC : Direction.DESC);
	}

	/**
	 * @return the orderSpecifier
	 */
	public List<OrderSpecifier<?>> getOrderSpecifiers() {
		return orderSpecifiers;
	}

	/**
	 * Returns a new {@link QSort} consisting of the {@link OrderSpecifier}s of the current {@code QSort} combined with
	 * the ones from the given {@code QSort}.
	 * 
	 * @param sort can be {@literal null}.
	 * @return
	 */
	public QSort and(QSort sort) {
		return sort == null ? this : and(sort.getOrderSpecifiers());
	}

	/**
	 * Returns a new {@link QSort} consisting of the {@link OrderSpecifier}s of the current {@link QSort} combined with
	 * the given ones.
	 * 
	 * @param orderSpecifiers must not be {@literal null} or empty.
	 * @return
	 */
	public QSort and(List<OrderSpecifier<?>> orderSpecifiers) {

		Assert.notEmpty(orderSpecifiers, "OrderSpecifiers must not be null or empty!");

		List<OrderSpecifier<?>> newOrderSpecifiers = new ArrayList<OrderSpecifier<?>>(this.orderSpecifiers);
		newOrderSpecifiers.addAll(orderSpecifiers);

		return new QSort(newOrderSpecifiers);
	}

	/**
	 * Returns a new {@link QSort} consisting of the {@link OrderSpecifier}s of the current {@link QSort} combined with
	 * the given ones.
	 * 
	 * @param orderSpecifiers must not be {@literal null} or empty.
	 * @return
	 */
	public QSort and(OrderSpecifier<?>... orderSpecifiers) {

		Assert.notEmpty(orderSpecifiers, "OrderSpecifiers must not be null or empty!");
		return and(Arrays.asList(orderSpecifiers));
	}
}
